@@ -14,13 +14,6 @@ bool epoll_check(void);
 extern "C" {
 #endif
 
-#ifdef HAVE_ACTSCHED
-void actsched_init(void);
-int  actsched_start(void);
-void actsched_stop(void);
-void actsched_restart_timer(void);
-#endif
-
 #ifdef USE_OPENSSL
 int  openssl_init(void);
 void openssl_close(void);
