@@ -208,18 +208,18 @@ int tls_alloc(struct tls **tlsp, enum tls_method method, const char *keyfile,
  * Set default locations for trusted CA certificates
  *
  * @param tls    TLS Context
- * @param capath Path to CA certificates
+ * @param cafile PEM file with CA certificates
  *
  * @return 0 if success, otherwise errorcode
  */
-int tls_add_ca(struct tls *tls, const char *capath)
+int tls_add_ca(struct tls *tls, const char *cafile)
 {
-	if (!tls || !capath)
+	if (!tls || !cafile)
 		return EINVAL;
 
 	/* Load the CAs we trust */
-	if (!(SSL_CTX_load_verify_locations(tls->ctx, capath, 0))) {
-		DEBUG_WARNING("Can't read CA list: %s\n", capath);
+	if (!(SSL_CTX_load_verify_locations(tls->ctx, cafile, NULL))) {
+		DEBUG_WARNING("Can't read CA file: %s\n", cafile);
 		ERR_clear_error();
 		return EINVAL;
 	}
@@ -735,6 +735,22 @@ int tls_srtp_keyinfo(const struct tls_conn *tc, enum srtp_suite *suite,
 		salt_size = 14;
 		break;
 
+#ifdef SRTP_AEAD_AES_128_GCM
+	case SRTP_AEAD_AES_128_GCM:
+		*suite = SRTP_AES_128_GCM;
+		key_size  = 16;
+		salt_size = 12;
+		break;
+#endif
+
+#ifdef SRTP_AEAD_AES_256_GCM
+	case SRTP_AEAD_AES_256_GCM:
+		*suite = SRTP_AES_256_GCM;
+		key_size  = 32;
+		salt_size = 12;
+		break;
+#endif
+
 	default:
 		return ENOSYS;
 	}
