@@ -27,6 +27,9 @@
 /* NOTE: shadow struct defined in tls_*.c */
 struct tls_conn {
 	SSL *ssl;
+#ifdef TLS_BIO_OPAQUE
+	BIO_METHOD *biomet;
+#endif
 	BIO *sbio_out;
 	BIO *sbio_in;
 	struct tcp_helper *th;
@@ -47,6 +50,12 @@ static void destructor(void *arg)
 
 		SSL_free(tc->ssl);
 	}
+
+#ifdef TLS_BIO_OPAQUE
+	if (tc->biomet)
+		BIO_meth_free(tc->biomet);
+#endif
+
 	mem_deref(tc->th);
 	mem_deref(tc->tcp);
 }
@@ -125,7 +134,29 @@ static long bio_ctrl(BIO *b, int cmd, long num, void *ptr)
 }
 
 
-#ifndef TLS_BIO_OPAQUE
+#ifdef TLS_BIO_OPAQUE
+
+static BIO_METHOD *bio_method_tcp(void)
+{
+	BIO_METHOD *method;
+
+	method = BIO_meth_new(BIO_TYPE_SOURCE_SINK, "tcp_send");
+	if (!method) {
+		DEBUG_WARNING("alloc: BIO_meth_new() failed\n");
+		ERR_clear_error();
+		return NULL;
+	}
+
+	BIO_meth_set_write(method, bio_write);
+	BIO_meth_set_ctrl(method, bio_ctrl);
+	BIO_meth_set_create(method, bio_create);
+	BIO_meth_set_destroy(method, bio_destroy);
+
+	return method;
+}
+
+#else
+
 static struct bio_method_st bio_tcp_send = {
 	BIO_TYPE_SOURCE_SINK,
 	"tcp_send",
@@ -138,6 +169,7 @@ static struct bio_method_st bio_tcp_send = {
 	bio_destroy,
 	0
 };
+
 #endif
 
 
@@ -347,6 +379,14 @@ int tls_start_tcp(struct tls_conn **ptc, struct tls *tls, struct tcp_conn *tcp,
 
 	tc->tcp = mem_ref(tcp);
 
+#ifdef TLS_BIO_OPAQUE
+	tc->biomet = bio_method_tcp();
+	if (!tc->biomet) {
+		err = ENOMEM;
+		goto out;
+	}
+#endif
+
 	err = ENOMEM;
 
 	/* Connect the SSL socket */
@@ -366,7 +406,7 @@ int tls_start_tcp(struct tls_conn **ptc, struct tls *tls, struct tcp_conn *tcp,
 
 
 #ifdef TLS_BIO_OPAQUE
-	tc->sbio_out = BIO_new(tls->method_tcp);
+	tc->sbio_out = BIO_new(tc->biomet);
 #else
 	tc->sbio_out = BIO_new(&bio_tcp_send);
 #endif
@@ -395,26 +435,3 @@ int tls_start_tcp(struct tls_conn **ptc, struct tls *tls, struct tcp_conn *tcp,
 
 	return err;
 }
-
-
-#ifdef TLS_BIO_OPAQUE
-
-BIO_METHOD *tls_method_tcp(void)
-{
-	BIO_METHOD *method;
-
-	method = BIO_meth_new(BIO_TYPE_SOURCE_SINK, "tcp_send");
-	if (!method) {
-		DEBUG_WARNING("alloc: BIO_meth_new() failed\n");
-		ERR_clear_error();
-		return NULL;
-	}
-
-	BIO_meth_set_write(method, bio_write);
-	BIO_meth_set_ctrl(method, bio_ctrl);
-	BIO_meth_set_create(method, bio_create);
-	BIO_meth_set_destroy(method, bio_destroy);
-
-	return method;
-}
-#endif
